@@ -36,17 +36,20 @@ The input L</url> to given as argument to the plugin need to point to a
 valid L<swagger|https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md>
 document.
 
-Every operation must have a "x-mojo-controller" specified,
-so this plugin knows where to look for the decamelized "operationId",
-which is used as method name.
+Every operation must have a "x-mojo-controller" specified, so this plugin
+knows where to look for the decamelized "operationId", which is used as
+method name. C<x-mojo-controller> can be defined on different levels
+and gets inherited unless defined more specific:
 
   ---
   swagger: 2.0
   basePath: /api
+  x-mojo-controller: MyApp::Controller::Default
   paths:
-    /foo:
+    /pets:
+      x-mojo-controller: MyApp::Controller::ForEveryHttpMethodUnderPets
       get:
-        x-mojo-controller: MyApp::Controller::Api
+        x-mojo-controller: MyApp::Controller::Petstore
         x-mojo-around-action: MyApp::authenticate_api_request
         operationId: listPets
         parameters: [ ... ]
@@ -75,11 +78,11 @@ and a callback. The callback should be called with a HTTP status code, and
 a data structure which will be validated and serialized back to the user
 agent.
 
-  package MyApp::Controller::Api;
+  package MyApp::Controller::Petstore;
 
   sub list_pets {
     my ($c, $input, $cb) = @_;
-    $c->$cb({foo => 123}, 200);
+    $c->$cb({limit => 123}, 200);
   }
 
 =head2 Protected API
@@ -113,9 +116,9 @@ L<relaxed placeholder|https://metacpan.org/pod/distribution/Mojolicious/lib/Mojo
   swagger: 2.0
   basePath: /api
   paths:
-    /foo:
+    /pets:
       get:
-        x-mojo-controller: MyApp::Controller::Api
+        x-mojo-controller: MyApp::Controller::Petstore
         operationId: listPets
         parameters:
         - name: ip
@@ -151,6 +154,17 @@ L</Swagger specification>, C<MyApp::authenticate_api_request>:
     );
   }
 
+C<x-mojo-around-action> is also inherited from most levels, meaning that you
+define it globally for your whole API if you like:
+
+  ---
+  x-mojo-around-action: MyApp::protect_any_resource
+  paths:
+    /pets:
+      x-mojo-around-action: MyApp::protect_any_method_under_foo
+      get:
+        x-mojo-around-action: MyApp::protect_just_this_resource
+
 This feature is EXPERIMENTAL and can change without notice.
 
 =head2 Stash variables
@@ -165,13 +179,25 @@ as C<swagger> from L<stash|Mojolicious/stash>. Example code:
     $c->$cb($c->stash('swagger')->pod->to_string, 200);
   }
 
+=head3 swagger_operation_spec
+
+The Swagger specification for the current
+L<operation object|https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#operationObject>
+is stored in the "swagger_operation_spec" stash variable.
+
+  sub list_pets {
+    my ($c, $args, $cb);
+    $c->app->log->info($c->stash("swagger_operation_spec")->{operationId});
+    ...
+  }
+
 =cut
 
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::JSON;
 use Mojo::Util 'decamelize';
-use JSON::Validator;
 use Swagger2;
+use Swagger2::SchemaValidator;
 use constant DEBUG => $ENV{SWAGGER2_DEBUG} || 0;
 
 =head1 ATTRIBUTES
@@ -183,7 +209,7 @@ Holds the URL to the swagger specification file.
 =cut
 
 has url => '';
-has _validator => sub { JSON::Validator->new; };
+has _validator => sub { Swagger2::SchemaValidator->new; };
 
 =head1 HELPERS
 
@@ -230,7 +256,7 @@ to render a JSON document, like this:
     "errors": [
       {
         "message": "is missing and it is required",
-        "path": "/foo"
+        "path": "/limit"
       },
       ...
     ]
@@ -328,10 +354,14 @@ sub register {
   }
 
   for my $path (keys %$paths) {
+    $paths->{$path}{'x-mojo-around-action'} ||= $swagger->api_spec->get('/x-mojo-around-action');
+    $paths->{$path}{'x-mojo-controller'}    ||= $swagger->api_spec->get('/x-mojo-controller');
+
     for my $http_method (grep { !/^x-/ } keys %{$paths->{$path}}) {
-      my $info       = $paths->{$path}{$http_method};
+      my $op_spec    = $paths->{$path}{$http_method};
       my $route_path = $path;
-      my %parameters = map { ($_->{name}, $_) } @{$info->{parameters} || []};
+      my %parameters = map { ($_->{name}, $_) } @{$op_spec->{parameters} || []};
+      my $route_name;
 
       $route_path =~ s/{([^}]+)}/{
         my $name = $1;
@@ -339,8 +369,16 @@ sub register {
         "($type$name)";
       }/ge;
 
-      warn "[Swagger2] Add route $http_method $route_path\n" if DEBUG;
-      $r->$http_method($route_path => $self->_generate_request_handler($route_path, $info));
+      $op_spec->{'x-mojo-around-action'} ||= $paths->{$path}{'x-mojo-around-action'};
+      $op_spec->{'x-mojo-controller'}    ||= $paths->{$path}{'x-mojo-controller'};
+      $route_name = decamelize(
+        join '::',
+        map { ucfirst $_ } $op_spec->{'x-mojo-controller'},
+        ($op_spec->{operationId} || $route_path)
+      );
+      $route_name =~ s/\W+/_/g;
+      $r->$http_method($route_path => $self->_generate_request_handler($route_path, $op_spec))->name($route_name);
+      warn "[Swagger2] Add route $http_method $route_path ($route_name)\n" if DEBUG;
     }
   }
 }
@@ -350,7 +388,7 @@ sub _generate_request_handler {
   my $op         = $config->{operationId} || $route_path;
   my $method     = decamelize(ucfirst $op);
   my $controller = $config->{'x-mojo-controller'} or _die($config, "x-mojo-controller is missing in the swagger spec");
-  my $defaults   = {};
+  my $defaults   = {swagger_operation_spec => $config};
   my $handler;
 
   $handler = sub {
