@@ -6,7 +6,7 @@ Swagger2 - Swagger RESTful API Documentation
 
 =head1 VERSION
 
-0.52
+0.53
 
 =head1 DESCRIPTION
 
@@ -60,8 +60,9 @@ use Mojo::URL;
 use Mojo::Util 'deprecated';
 use File::Basename ();
 use File::Spec;
+use Swagger2::SchemaValidator;
 
-our $VERSION = '0.52';
+our $VERSION = '0.53';
 
 # Should be considered internal
 our $SPEC_FILE = File::Spec->catfile(File::Basename::dirname(__FILE__), 'Swagger2', 'schema.json');
@@ -140,8 +141,7 @@ sub tree {
 has _specification => sub { shift->_validator->schema($SPEC_FILE)->schema };
 
 has _validator => sub {
-  require JSON::Validator;
-  JSON::Validator->new->cache_dir($ENV{SWAGGER2_CACHE_DIR}
+  Swagger2::SchemaValidator->new->cache_dir($ENV{SWAGGER2_CACHE_DIR}
       || File::Spec->catdir(File::Basename::dirname(__FILE__), qw( Swagger2 public cache )));
 };
 
